package com.redstrate.watersymbol;

import com.badlogic.gdx.maps.MapObjects;
import com.badlogic.gdx.maps.objects.RectangleMapObject;
import com.badlogic.gdx.maps.tiled.TiledMap;
import com.badlogic.gdx.maps.tiled.TiledMapTileLayer;
import com.badlogic.gdx.math.Intersector;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector2;

import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;

/*
This is a direct port of the A* pathing implementation in Realm of Chaos.
 */
public class AStar {
    static class Node {
        Vector2 position;
        int g = 0;
        int h = 0;
        int f = 0;

        Node parent;
    }

    private static final int maxOpenNodes = 200;

    public static ArrayList<Vector2> path(Vector2 start, Vector2 end, List<Vector2> collisionPoints, TiledMap map) {
        Node startNode = new Node();
        startNode.position = start;

        Node endNode = new Node();
        endNode.position = end;

        ArrayList<Node> openList = new ArrayList<>();
        openList.add(startNode);

        ArrayList<Node> closedList = new ArrayList<>();
        while(!openList.isEmpty() && openList.size() < maxOpenNodes) {
            Node currentNode = openList.get(0);
            int currentIndex = 0;
            for(int i = 0; i < openList.size(); i++) {
                if(openList.get(i).f < currentNode.f) {
                    currentNode = openList.get(i);
                    currentIndex = i;
                }
            }

            openList.remove(currentIndex);
            closedList.add(currentNode);

            // check if we found the goal
            if(currentNode.position.epsilonEquals(endNode.position)) {
                ArrayList<Vector2> path = new ArrayList<>();
                Node current = currentNode;
                while(current != null) {
                    path.add(current.position);
                    current = current.parent;
                }
                
                return path;
            }

            ArrayList<Node> children = new ArrayList<>();

            ArrayList<Vector2> adjacentPositions = new ArrayList<>();
            adjacentPositions.add(new Vector2(0, -1));
            adjacentPositions.add(new Vector2(0, 1));
            adjacentPositions.add(new Vector2(-1, 0));
            adjacentPositions.add(new Vector2(1, 0));
            adjacentPositions.add(new Vector2(-1, -1));
            adjacentPositions.add(new Vector2(-1, 1));
            adjacentPositions.add(new Vector2(1, -1));
            adjacentPositions.add(new Vector2(1, 1));
            for (Vector2 position : adjacentPositions) {
                Node newNode = new Node();
                newNode.position = new Vector2(currentNode.position.x + position.x, currentNode.position.y + position.y);
                newNode.parent = currentNode;

                boolean shouldIgnore = false;
                for(Vector2 collisionPosition : collisionPoints) {
                    if(collisionPosition.epsilonEquals(newNode.position) && !collisionPosition.epsilonEquals(end)) {
                        shouldIgnore = true;
                    }
                }

                TiledMapTileLayer.Cell cell = null;
                try {
                    TiledMapTileLayer layer = (TiledMapTileLayer) map.getLayers().get(0);
                    cell = layer.getCell((int) (newNode.position.x), (int) (newNode.position.y));
                } catch (Exception e) {
                    e.printStackTrace();
                }

                if(!cell.getTile().getProperties().get("walkable", Boolean.class))
                    shouldIgnore = true;

                if(!shouldIgnore) {
                    children.add(newNode);
                }
            }

            for(Node child : children) {
                boolean shouldContinue = false;
                for(Node closedChild : closedList) {
                    if(child.position.epsilonEquals(closedChild.position)) {
                        shouldContinue = true;
                    }
                }

                if(shouldContinue)
                    continue;

                child.g = currentNode.g + 1;
                child.h = (int) (Math.pow(child.position.x - endNode.position.x, 2) + Math.pow(child.position.y - endNode.position.y, 2));
                child.f = child.g + child.h;

                for(Node openNode : openList) {
                    if(child.position.epsilonEquals(openNode.position) && child.g > openNode.g) {
                        shouldContinue = true;
                    }
                }

                if(shouldContinue)
                    continue;

                openList.add(child);
            }
        }

        return null;
    }
}
