package com.redstrate.watersymbol.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.GlyphLayout;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.utils.ScreenUtils;
import com.redstrate.watersymbol.WaterSymbol;

public class MainMenuScreen implements Screen {
    OrthographicCamera camera;

    GlyphLayout titleLayout;

    Texture logoTexture;
    Texture diamondTexture;
    Sprite backgroundSprite;

    public MainMenuScreen(WaterSymbol game) {
        this.game = game;

        camera = new OrthographicCamera();
        camera.setToOrtho(false, 800, 800);

        logoTexture = new Texture("logo.png");

        diamondTexture = new Texture("diamond.png");
        diamondTexture.setWrap(Texture.TextureWrap.Repeat, Texture.TextureWrap.Repeat);

        backgroundSprite = new Sprite(diamondTexture);
        backgroundSprite.setColor(0.2f, 0.2f, 0.3f, 1.0f);

        titleLayout = new GlyphLayout(game.font, "PRESS ANY BUTTON");
    }

    @Override
    public void show() {

    }

    @Override
    public void render(float delta) {
        ScreenUtils.clear(0.02f, 0.01f, 0.2f, 1);

        camera.update();
        game.batch.setProjectionMatrix(camera.combined);

        game.batch.begin();

        backgroundSprite.setPosition(camera.position.x - camera.viewportWidth / 2, camera.position.y - camera.viewportHeight / 2);
        backgroundSprite.scroll(delta, delta);
        backgroundSprite.draw(game.batch);

        game.batch.draw(logoTexture, camera.position.x - (logoTexture.getWidth() / 2), camera.position.y - 40);

        game.font.draw(game.batch, titleLayout, camera.position.x - (titleLayout.width / 2), camera.position.y - 60);

        game.batch.end();

        if(Gdx.input.isButtonJustPressed(Input.Buttons.LEFT)) {
            game.setScreen(new GameScreen(game));
            dispose();
        }
    }

    @Override
    public void resize(int width, int height) {
        camera.viewportWidth = 430;
        camera.viewportHeight = 430 * ((float)height / (float)width);

        backgroundSprite.setSize(camera.viewportWidth, camera.viewportHeight);
    }

    @Override
    public void pause() {

    }

    @Override
    public void resume() {

    }

    @Override
    public void hide() {

    }

    @Override
    public void dispose() {

    }

    private final WaterSymbol game;
}
